library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_misc.all;
use ieee.numeric_std.all;

entity dummy_clk_gen is
  generic (
    g_NUM      : integer := 16;
    g_PORT_NUM : integer := 1;
    g_HI_Z     : boolean := false);
  port (
    rst_n_i : in  std_logic;
    clk_i   : in  std_logic;
    gen_o   : out std_logic_vector(g_NUM-1 downto 0)
  );
end dummy_clk_gen;

architecture rtl of dummy_clk_gen is

  constant c_CLK_I   : integer := 25000000;
  constant c_PERIOD  : integer := c_CLK_I / g_PORT_NUM;
  constant c_PH_STEP : integer := c_PERIOD / g_NUM;

  type t_cntrs is array (natural range <>) of unsigned(24 downto 0);
  signal cnt : t_cntrs(0 to g_NUM-1);

  signal gen_out : std_logic_vector(g_NUM-1 downto 0);

begin

  gen_o <= gen_out;

  GEN_OUTS: for I in 0 to g_NUM-1 generate
    process(clk_i)
    begin
      if rising_edge(clk_i) then
        if rst_n_i = '0' then
          gen_out(I) <= '0';
          cnt(I)     <= to_unsigned(I*c_PH_STEP, 25);
        else
          if cnt(I) < c_PERIOD/2 then
            cnt(I)     <= cnt(I) + 1;
            gen_out(I) <= '0';
          elsif cnt(I) < c_PERIOD and g_HI_Z = false then
            cnt(I)     <= cnt(I) + 1;
            gen_out(I) <= '1';
          elsif cnt(I) < c_PERIOD and g_HI_Z = true then
            cnt(I)     <= cnt(I) + 1;
            gen_out(I) <= 'Z';
          else
            cnt(I)     <= (others=>'0');
          end if;
        end if;
      end if;
    end process;
  end generate;

end rtl;
